import netCDF4 as nc4
import numpy.ma as ma
import numpy as np

vars_calc = ['top_melt','fcondtop','fcondbot','ocean_heat_flux']
in_file = # Set input file: a netCDF4 file containing a dataset of monthly mean energy flux data from IMBs
writefile = # Set location of output file where the tables will be written
ncid = nc4.Dataset(in_file)

salinities = {'top_melt': 0, 'fcondtop': 1, 'fcondbot': 4., 'ocean_heat_flux':4}

means = {}
stds = {}
dists = {}
npts = {}


for varname in vars_calc:
    means[varname] = ma.masked_array(np.zeros((3,12)),mask=np.zeros((3,12),dtype='bool'))
    stds[varname] = ma.masked_array(np.zeros((3,12)),mask=np.zeros((3,12),dtype='bool'))
    dists[varname] = {0:[],1:[],2:[]}
    npts[varname] = np.zeros((3,12),dtype='int')
    isalin = np.where(ncid.variables['salinity'][:] == salinities[varname])[0]
    for month in range(1,13):
        index = np.where(ncid.variables['month'][:] == month)

        if index[0].shape[0] > 0:
            dist_2D = ncid.variables[varname][index[0],:]
            distribution = []
            for irec in range(dist_2D.shape[0]):
                item_array = dist_2D[irec,:]
                if item_array.mask[isalin] == False:
                    distribution.append(
                        item_array.data[isalin])
                elif False in item_array.mask:
                    distribution.append(
                        item_array.data[np.argmax(item_array.mask)-1])
                else:
                    pass

            distribution = np.array(distribution)
            dists[varname][0].append(distribution)
            means[varname][0,month-1] = np.mean(distribution)
            stds[varname][0,month-1] = np.std(distribution)
            npts[varname][0,month-1] = distribution.shape[0]
            
    
        for region_code in range(1,3):
            index = np.where(np.logical_and(ncid.variables['month'][:] == month,\
                         ncid.variables['region_code'][:] == region_code))

            if index[0].shape[0] > 0:
                dist_2D = ncid.variables[varname][index[0],:]
                distribution = []
                for irec in range(dist_2D.shape[0]):
                    item_array = dist_2D[irec,:]
                    if item_array.mask[isalin] == False:
                        distribution.append(
                            item_array.data[isalin])
                    elif False in item_array.mask:
                        distribution.append(
                            item_array.data[np.argmax(item_array.mask)-1])
                    else:
                        pass
    
                distribution = np.array(distribution)
                dists[varname][region_code].append(distribution)
                means[varname][region_code,month-1] = np.mean(distribution)
                stds[varname][region_code,month-1] = np.std(distribution)
                npts[varname][region_code,month-1] = distribution.shape[0]

region_keys = {0:'All',1:'North_Pole',2:'Beaufort_Sea'}

fileh = open(writefile,'w')

titles1 = [['Month']] + [[varname,''] for varname in vars_calc]
titles1 = [item for sublist in titles1 for item in sublist]

titles2 = [''] + ['Mean','Stddev'] * 4

for region_code in range(3):
    fileh.write('\n')
    fileh.write(region_keys[region_code]+'\n')
    fileh.write(('{:4s}'+('{:6s}'*8)+'\n').format(*titles1))
    fileh.write(('{:4s}'+('{:12d}'*4)+'\n').format('npt',*[np.sum(npts[varname][region_code,:]) 
        for varname in vars_calc]))
    fileh.write(('{:4s}'+('{:6s}'*8)+'\n').format(*titles2))
    for month in range(1,13):
        data_out = [[means[varname][region_code,month-1],stds[varname][region_code,month-1]] for varname in vars_calc]
        data_out_flatten = [item for sublist in data_out for item in sublist]
        fileh.write(('{:4d}'+('{:6.1f}'*8)+'\n').format(month,*data_out_flatten))

fileh.close()
