import iris
import numpy as np
import matplotlib.pyplot as plt
import pylab
import netCDF4 as nc4
import glob
import timemod
import regions
import numpy.ma as ma
import matplotlib.pyplot as plt
import itd_functions

ice_thickness_bins = np.arange(0,4.1,.2)
snow_depth_bins = np.arange(0,.61,.05)

month_set = [11,12,1,2,3]
region_names = ['North_Pole','Beaufort_Sea']
region_codes = {'North_Pole':1,'Beaufort_Sea':2}

buoy_file = '/data/users/hadax/PhD/Buoys/new_output/v5_roc_polyfit_zint_height/time_mean_variables/all_variables_monthly_mu.nc'
buoy_ncid = nc4.Dataset(buoy_file)
buoy_months = buoy_ncid.variables['month']

outfile = # Set location of output file

model_dir_cat = # Set location of model category ice area and thickness data
model_dir_flux = # Set location of model ice energy flux data
model_dir_state = # Set location of model ice concentration and thickness, 
                  # and snow thickness, data


ice_files_cat = glob.glob(model_dir_cat + '?????o.pm?????.pp')
cubes = iris.load(ice_files_cat)
scs = [ccube.attributes['STASH'] for ccube in cubes]
aicen = cubes[scs.index((2,0,223))]
vicen = cubes[scs.index((2,0,224))]

ice_files_flux = glob.glob(model_dir_flux + '?????o.pm?????.pp')
fcondtop = iris.load_cube(ice_files_flux,'downward_heat_flux_in_sea_ice')

ice_files_state = glob.glob(model_dir_state + '?????o.pm?????.pp')
aice = iris.load_cube(ice_files_state,'sea_ice_area_fraction')
vice = iris.load_cube(ice_files_state,'sea_ice_thickness')
hsno = iris.load_cube(ice_files_state,'surface_snow_thickness')

fcondtop_l = fcondtop / aice
hicen = vicen / aicen
hice = vice / aice

hicen_rcp_weight = aicen / hicen
hice_hm = aice / hicen_rcp_weight.collapsed(['pseudo_level'],iris.analysis.SUM)

xl = .05
yb = .55
xw = .4
yw = .4
xs = .5
ys = .5

letters = ['(a)','(b)','(c)','(d)']

figure = plt.figure(figsize=(12,12))

for (iregion,region_name) in enumerate(region_names):
    region_code_logical = buoy_ncid.variables['region_code'][:] == \
     region_codes[region_name]
    buoy_months_logical = np.array([month in month_set for month in 
                          buoy_months])
    buoy_index = np.where(np.logical_and(buoy_months_logical,
                                         region_code_logical))
    ice_thickness_imb = buoy_ncid.variables['ice_thickness'][buoy_index[0],0]
    fcondtop_imb = buoy_ncid.variables['fcondtop'][buoy_index[0],2]
    fcondbot_imb = buoy_ncid.variables['fcondbot'][buoy_index[0],8]
    snow_depth_imb = buoy_ncid.variables['snow_thickness'][buoy_index[0],0]

    months_ice = timemod.tmonth(hice_hm)
    months_snow = timemod.tmonth(hsno)
    index_m_ice = np.array([np.where(months_ice == mr)[0][0] 
        for mr in month_set])
    index_m_snow = np.array([np.where(months_snow == mr)[0][0] 
        for mr in month_set])

    hsno_r = hsno[index_m_snow,:,:]
    regions.reduce_to_region(hsno_r,region_name,'hadgom')
    ipts = np.where(hsno_r.data.mask == False)
    hsnopts = hsno.data[ipts]

    hice_r_hm = hice_hm[index_m_ice,:,:]
    regions.reduce_to_region(hice_r_hm,region_name,'hadgom')
    ipts = np.where(hice_r_hm.data.mask == False)
    hicepts_hm = hice_r_hm.data.data[ipts]

    fcondtop_dists_model_ice, fcondtop_model_ice_i = itd_functions.bin_by_hice(
        fcondtop_l[index_m_ice,:,:].data[ipts],hicepts_hm,ice_thickness_bins)
    fcondtop_dists_imb_ice, fcondtop_imb_ice_i = itd_functions.bin_by_hice(
        fcondtop_imb,ice_thickness_imb,ice_thickness_bins)
    fcondbot_dists_imb_ice, fcondbot_imb_ice_i = itd_functions.bin_by_hice(
        fcondbot_imb,ice_thickness_imb,ice_thickness_bins)

    fcondtop_model_ice_plot = [fcondtop_dists_model_ice[ii] for ii in range(len(fcondtop_dists_model_ice)) if ii in fcondtop_model_ice_i[0]]
    fcondtop_imb_ice_plot = [fcondtop_dists_imb_ice[ii] for ii in range(len(fcondtop_dists_imb_ice)) if ii in fcondtop_imb_ice_i[0]]
    fcondbot_imb_ice_plot = [fcondbot_dists_imb_ice[ii] for ii in range(len(fcondbot_dists_imb_ice)) if ii in fcondbot_imb_ice_i[0]]

    fcondtop_dists_model_snow, fcondtop_model_snow_i = itd_functions.bin_by_hice(
        fcondtop_l[index_m_snow,:,:].data[ipts],hsnopts,snow_depth_bins)
    fcondtop_dists_imb_snow, fcondtop_imb_snow_i = itd_functions.bin_by_hice(
        fcondtop_imb,snow_depth_imb,snow_depth_bins)
    fcondbot_dists_imb_snow, fcondbot_imb_snow_i = itd_functions.bin_by_hice(
        fcondbot_imb,snow_depth_imb,snow_depth_bins)

    fcondtop_model_snow_plot = [fcondtop_dists_model_snow[ii] for ii in range(len(fcondtop_dists_model_snow)) if ii in fcondtop_model_snow_i[0]]
    fcondtop_imb_snow_plot = [fcondtop_dists_imb_snow[ii] for ii in range(len(fcondtop_dists_imb_snow)) if ii in fcondtop_imb_snow_i[0]]
    fcondbot_imb_snow_plot = [fcondbot_dists_imb_snow[ii] for ii in range(len(fcondbot_dists_imb_snow)) if ii in fcondbot_imb_snow_i[0]]


    means_model_ice = np.array([np.mean(dist) for dist in fcondtop_dists_model_ice])
    means_imb_ice = np.array([np.mean(dist) for dist in fcondtop_dists_imb_ice])
    means_imb_ice_bot = np.array([np.mean(dist) for dist in fcondbot_dists_imb_ice])

    means_model_snow = np.array([np.mean(dist) for dist in fcondtop_dists_model_snow])
    means_imb_snow = np.array([np.mean(dist) for dist in fcondtop_dists_imb_snow])
    means_imb_snow_bot = np.array([np.mean(dist) for dist in fcondbot_dists_imb_snow])

    xi = 0
    yi = iregion
    axis_ice = figure.add_axes([xl+xs*xi,yb-ys*yi,xw,yw])
    xpos_ice = (ice_thickness_bins[1:] + ice_thickness_bins[:-1]) / 2.
    width_ice = .04
    offset_ice = .05


    bp_model = axis_ice.boxplot(fcondtop_model_ice_plot,positions = xpos_ice[fcondtop_model_ice_i]-offset_ice,widths = width_ice,\
       whis=10.)
    bp_imb = axis_ice.boxplot(fcondtop_imb_ice_plot,positions = xpos_ice[fcondtop_imb_ice_i],widths = width_ice,\
       whis=10.)
    bp_imb_bot = axis_ice.boxplot(fcondbot_imb_ice_plot,positions = xpos_ice[fcondbot_imb_ice_i]+offset_ice,widths = width_ice,\
       whis=10.)

    for item in ['boxes','whiskers','fliers','means','medians','caps']:
        plt.setp(bp_model[item],color='r')
        plt.setp(bp_imb[item],color='k')
        plt.setp(bp_imb_bot[item],color='#004090')

    axis_ice.plot(xpos_ice[fcondtop_model_ice_i]-offset_ice,means_model_ice[fcondtop_model_ice_i],label='HadGEM2-ES',\
             linestyle=' ',marker='o',markerfacecolor='r',markeredgecolor='k')
    axis_ice.plot(xpos_ice[fcondtop_imb_ice_i],means_imb_ice[fcondtop_imb_ice_i],label='IMBs - Top',\
             linestyle=' ',marker='o',markerfacecolor='k',markeredgecolor='k')
    axis_ice.plot(xpos_ice[fcondbot_imb_ice_i]+offset_ice,means_imb_ice_bot[fcondbot_imb_ice_i],label='IMBs - Base',\
             linestyle=' ',marker='o',markerfacecolor='#004090',
             markeredgecolor='k')

    ylim = axis_ice.get_ylim()
    axis_ice.set_xticks(np.arange(0,4.01,1))
    axis_ice.set_xticklabels(np.arange(0,4.01,1))
    for ibin in ice_thickness_bins:
        axis_ice.plot([ibin]*2,ylim,color='#e0e0e0',linestyle='--')


    axis_ice.set_xlabel('Ice thickness (m)')
    axis_ice.set_ylim(ylim)
    xlim = (np.min(ice_thickness_bins),np.max(ice_thickness_bins))
    axis_ice.set_xlim(xlim)
    axis_ice.set_title(letters[iregion*2])

    xi = 1
    yi = iregion
    axis_snow = figure.add_axes([xl+xs*xi,yb-ys*yi,xw,yw])
    xpos_snow = (snow_depth_bins[1:] + snow_depth_bins[:-1]) / 2.
    width_snow = .01
    offset_snow = .012

    bp_model = axis_snow.boxplot(fcondtop_model_snow_plot,positions = xpos_snow[fcondtop_model_snow_i]-offset_snow,widths = width_snow,\
       whis=10.)
    bp_imb = axis_snow.boxplot(fcondtop_imb_snow_plot,positions = xpos_snow[fcondtop_imb_snow_i],widths = width_snow,\
       whis=10.)
    bp_imb_bot = axis_snow.boxplot(fcondbot_imb_snow_plot,positions = xpos_snow[fcondbot_imb_snow_i]+offset_snow,widths = width_snow,\
       whis=10.)

    for item in ['boxes','whiskers','fliers','means','medians','caps']:
        plt.setp(bp_model[item],color='r')
        plt.setp(bp_imb[item],color='k')
        plt.setp(bp_imb_bot[item],color='#004090')

    axis_snow.plot(xpos_snow[fcondtop_model_snow_i]-offset_snow,means_model_snow[fcondtop_model_snow_i],label='HadGEM2-ES',\
             linestyle=' ',marker='o',markerfacecolor='r',markeredgecolor='k')
    axis_snow.plot(xpos_snow[fcondtop_imb_snow_i],means_imb_snow[fcondtop_imb_snow_i],label='IMBs - Top',\
             linestyle=' ',marker='o',markerfacecolor='k',markeredgecolor='k')
    axis_snow.plot(xpos_snow[fcondbot_imb_snow_i]+offset_snow,means_imb_snow_bot[fcondbot_imb_snow_i],label='IMBs - Base',\
             linestyle=' ',marker='o',markerfacecolor='#004090',
             markeredgecolor='k')

    ylim = axis_snow.get_ylim()
    axis_snow.set_xticks(np.arange(0,0.61,.05))
    axis_snow.set_xticklabels(['{:4.2f}'.format(number) for number in np.arange(0,0.61,.05)])
    for ibin in snow_depth_bins:
        axis_snow.plot([ibin]*2,ylim,color='#e0e0e0',linestyle='--')

    axis_snow.set_xlabel('Snow depth (m)')
    axis_snow.set_ylim(ylim)

    for axis in (axis_ice,axis_snow):
        axis.set_ylabel('Conductive flux ($Wm^{-2}$)')
        axis.plot(xlim,[0,0],color='k',linewidth=2)

    xlim = (np.min(snow_depth_bins),np.max(snow_depth_bins))
    axis_snow.set_xlim(xlim)
    axis_snow.set_title(letters[iregion*2+1])

axis.legend()

pylab.savefig(outfile)
