#This routine requires use of the open source iris and cartopy libraries

import cartopy.crs as ccrs
import matplotlib.pyplot as plt
import numpy as np
import sys
sys.path.insert(0,) # Insert reference to location of IMB basic processing code
                    # which is available at 
                    # https://github.com/sammiebuzzard/IMBS_MO
import buoys
import cartopy
import copy
import pylab
import os
import netCDF4 as nc4
import glob
import iris
import datetime as dt
from matplotlib.font_manager import FontProperties

font_large = FontProperties(size=18)

outfile = # Set location of output file
workdir = # Location to save data calculated during routine execution

regions_dir = # Set location of directory containing netCDF4 files with a masked array corresponding to the North Pole and Beaufort Sea regions respectively
regions_plot = ['North_Pole','Beaufort_Sea']

buoylist = buoys.buoylist()
nps = ccrs.NorthPolarStereo()
pc = ccrs.PlateCarree()

colours = ['#0000ff','#5050a0','#a0a050','#ffff00','#ffa000','#ff5000',\
           '#ff0000','#c00030','#a00070','#b000b0','#7000a0','#3000c0']


read_threshold = 102

xl = .05
xw = .9

figure = plt.figure(figsize=(11,12))
axis = figure.add_axes([xl,.16,xw,.8],projection=nps)

for (ibuoy,buoy_name) in enumerate(buoylist):
    savedirf = savedir + buoy_name + '/'
    if not os.path.exists(savedirf):
        os.makedirs(savedirf)
    print buoy_name

    if ibuoy > read_threshold:
        buoy = buoys.buoy(buoy_name)
        latitude = buoy.extract_data('latitude')
        longitude = buoy.extract_data('longitude')

        if len(latitude.dates()) > 0 and len(longitude.dates()) > 0:
            prev_key = (0,0)
            latpts = {prev_key:[]}
            lonpts = {prev_key:[]}

            for date in latitude.dates():
                if date in longitude.dates():
                    key = (date.month,date.year)
                    if not key in latpts.keys():
                        latpts[key] = []
                        latpts[prev_key].append(latitude.data_list[date])

                    latpts[key].append(latitude.data_list[date])
                    if not key in lonpts.keys():
                        lonpts[key] = []
                        lonpts[prev_key].append(longitude.data_list[date])

                    lonpts[key].append(longitude.data_list[date])
                    prev_key = copy.copy(key)

            for key in latpts.keys():

                axis.plot(lonpts[key],latpts[key],color=colours[key[0]-1],transform=pc)

            axis.add_feature(cartopy.feature.LAND)
            axis.set_extent(np.array([-1.,1.,-1.,1.])*3.e6,crs=nps)
            axis.set_title(buoy_name)

            outfile = workdir + buoy_name + '.png'
            pylab.savefig(outfile)

            for key in latpts.keys():
                dlat = np.array(latpts[key])
                dlon = np.array(lonpts[key])
                savefilename = str(key[0]) + '.' + str(key[1]) + '.nc'
                savefile = savedirf + savefilename
                ncid = nc4.Dataset(savefile,'w')
                ncid.createDimension('t',dlat.shape[0])
                lat_var = ncid.createVariable('latitude','float',('t',))
                lon_var = ncid.createVariable('longitude','float',('t',))
                lat_var[:] = dlat
                lon_var[:] = dlon
                ncid.close()
        
    else:
        readfiles = glob.glob(savedirf+'*.nc')
        filenames = [rfile.split('/')[-1] for rfile in readfiles]
        months = [int(fn.split('.')[0]) for fn in filenames]
        years = [int(fn.split('.')[1]) for fn in filenames]
        
        latpts = {}
        lonpts = {}
        for (fn,month,year) in zip(filenames,months,years):
            ncid = nc4.Dataset(savedirf+fn)
            key = (month,year)
            latpts[key] = ncid.variables['latitude'][:]
            lonpts[key] = ncid.variables['longitude'][:]
            if np.where(lonpts[key] > 90.)[0].shape[0] > 0 and \
               np.where(lonpts[key] < -90.)[0].shape[0] > 0:
                index = np.where(lonpts[key] < -90.)
                lonpts[key][index] = lonpts[key][index] + 360.
        
            axis.plot(lonpts[key],latpts[key],color=colours[key[0]-1],\
                 transform=pc)
    
axis.add_feature(cartopy.feature.LAND)
axis.add_feature(cartopy.feature.COASTLINE)
axis.set_extent(np.array([-1.,1.,-1.,1.])*3.e6,crs=nps)

for region in regions_plot:
    rfile = regions_dir + region + '.hadgom.nc'
    rcube = iris.load_cube(rfile)
    
    mask_array = rcube.data.mask.astype('int')
    axis.contour(rcube.coord('longitude').points,rcube.coord('latitude').points,mask_array,\
                 levels = [-.5,.5,1.5],transform = pc, colors = 'k', linewidths=2)

ax_cl = figure.add_axes([xl,.1,xw,.03])
for month in range(1,13):
    ax_cl.fill_between([month-1,month],[0,0],[1,1],color=colours[month-1])
    month_name = dt.date(1980,month,1).strftime('%b')
    ax_cl.text(month-.5,-.7,month_name,ha='center',fontproperties=font_large)
    axis.plot([month,month],[0,1],color='k')

for label in ax_cl.get_xticklabels() + ax_cl.get_yticklabels():
    label.set_visible(False)

axis.text(65.,83.7,'North\nPole',transform=pc,fontproperties=font_large)
axis.text(178.,76.,'Beaufort\nSea',transform=pc,fontproperties=font_large)

pylab.savefig(outfile_eps)
pylab.savefig(outfile_png)
