import sys
sys.path.insert(0,) # Insert reference to location of IMB basic processing code
                    # which is available at 
                    # https://github.com/sammiebuzzard/IMBS_MO
import buoys
import datetime as dt
import matplotlib.pyplot as plt
import pylab
import tprof
import numpy as np
import matplotlib

outfile = # insert output file name

elev_labels = ['surface_rt','interface_rt','bottom_rt']
example_date = dt.datetime(2013,2,12,8,0)
buoy_name = '2012J'

aa = buoys.buoy(buoy_name)
aa.extract_data('surface')
aa.extract_data('bottom')
aa.interface_from_surface()
aa.process_temp()
aa.data['surface_rt'] = aa.data['surface'].regularise_temp(aa.temp.dates())
aa.data['interface_rt'] = aa.data['interface_r'].regularise_temp(aa.temp.dates())
aa.data['bottom_rt'] = aa.data['bottom'].regularise_temp(aa.temp.dates())
aa.new_sfc_temp_series(0.)
aa.new_sfc_temp_series(-.5)
zpoints = aa.temp.zpoints()

profile = aa.temp.mprofile_set[example_date]
profile_adj = tprof.snow_adjusted_zprof(aa,example_date)

int_offset = .05
int_pos = aa.data['interface_rt'].data_list[example_date] - int_offset
int_temp = tprof.estimate_prof(zpoints,profile,int_pos)

bvars = ['surface_temp_-0.500','surface_temp_0.000']
zint50 = [ii + aa.data['surface_rt'].estimate(example_date) for ii in \
    [-.5,0]]

top50_section = tprof.complete_zprof(aa,example_date,zint50,boundary_vars=bvars)
top50_section_adj = tprof.complete_zprof(aa,example_date,zint50,\
        boundary_vars=None,adjust_for_snow=True)

grad = np.polyfit(top50_section[1],top50_section[0],1)
grad_adj = np.polyfit(top50_section_adj[1],top50_section_adj[0],1)

elevation_values = {}
for elev_label in elev_labels:
    elevation_values[elev_label] = aa.data[elev_label].estimate(example_date)

titles = ['Without adjustment','With adjustment']
xlim = [-32,2]
ylim = [-2.8,0.8]
figure = plt.figure(figsize=(12,8))
ax1 = figure.add_axes([.05,.1,.4,.8])
ax2 = figure.add_axes([.55,.1,.4,.8])

for ax in [ax1,ax2]:
    ax.fill_between(xlim,[elevation_values['surface_rt']]*2,\
        [elevation_values['surface_rt']-.5]*2,color='#e0e0ff')

ax1.plot(profile,aa.temp.zpoints(),color='r',marker='o')
ax1.plot((np.array(zint50)-grad[1])/grad[0],np.array(zint50),linestyle='--',color='k')

longlabel_x = -15
longlabel_y = -.6
ax1.arrow(longlabel_x,longlabel_y,(int_temp - longlabel_x),\
          (int_pos - longlabel_y),linestyle='--',color='k',\
                                         length_includes_head=True,\
              head_width=.04,head_length=.8)
ax1.text(longlabel_x,longlabel_y,'The temperature at a level\n5cm below the interface\nis used as a reference',ha='right',va='top')

ax2.plot(profile,aa.temp.zpoints(),color='#ffc0c0',marker='o')
ax2.plot(profile_adj,aa.temp.zpoints(),color='#a04000',marker='*')
ax2.plot((np.array(zint50)-grad_adj[1])/grad_adj[0],np.array(zint50),linestyle='--',color='k')

line, = ax2.plot([int_temp]*2,\
        [int_pos,zpoints[0]],color='r')
line.set_dashes([1,2])

for level in range(0,5):
    ax2.arrow(profile[level],\
              zpoints[level],profile_adj[level]-profile[level],0,\
              color='k',linestyle='--',length_includes_head=True,\
              head_width=.04,head_length=.8)

demo_level = .55
ax2.plot([profile_adj[0],profile[0],int_temp],[demo_level]*3,color='k')
for xpt in [profile_adj[0],profile[0],int_temp]:
    ax2.plot([xpt]*2,[demo_level-.04,demo_level+.04],color='k')

ax2.text((profile_adj[0] + profile[0])/2.,demo_level+.03,\
         r'$\alpha k_{ice}$',ha='center')
ax2.text((int_temp + profile_adj[0])/2.,demo_level+.06,\
         r'$\alpha k_{snow}$',ha='center')

for (iax,ax) in enumerate((ax1,ax2)):
    for elev_label in elev_labels:
        ax.plot(xlim,[elevation_values[elev_label]]*2,color='#0000a0')
        ax.text(xlim[1],elevation_values[elev_label]+.05,elev_label[:-3].capitalize(),ha='right')

    ax.plot(xlim,[elevation_values['surface_rt']-.5]*2,color='#0000a0',\
        linestyle='--')
    ax.text(xlim[1],elevation_values['surface_rt']-.5+.05,'50cm below surface',\
        ha='right')
    ax.plot([int_temp],[int_pos],markerfacecolor='r',markeredgecolor='k',marker='s')

    ax.set_xlim(xlim)
    ax.set_ylim(ylim)
    ax.set_title(titles[iax])


figure.suptitle('Taking linear fits for top conductive flux:')
pylab.savefig(outfile)
