import netCDF4 as nc4
import numpy as np
import matplotlib.pyplot as plt
import pylab

source_file =  # Set input file: a netCDF4 file containing a dataset of monthly mean energy flux data from IMBs. 
outfile = # Set location of output file 
varnames = ['top_melt','fcondtop','fcondbot','ocean_heat_flux']
var_titles = {'top_melt':'Top melting flux',
              'fcondtop':'Top conductive flux',
              'fcondbot':'Basal conductive flux',
              'ocean_heat_flux':'Ocean heat flux',
              'ice_thickness':'Ice thickness',
              'snow_thickness':'Snow depth'}
salinities = {'top_melt':0.,
              'fcondtop':1.,
              'fcondbot':4.,
              'ocean_heat_flux':4.,
              'ice_thickness':0,
              'snow_thickness':0}
ylims = {'top_melt':[-5,80],
         'fcondtop':[-60,15],
         'fcondbot':[-60,15],
         'ocean_heat_flux':[-20,75],
         'ice_thickness':[0,4],
         'snow_thickness':[0,0.6]}

dividers = [1993,2007,2013,2016]
colours = {1993:'b',2007:'#808080',2013:'r'}

ncid = nc4.Dataset(source_file)

nperiods = len(dividers)-1
means = {}
stddevs = {} 
npts = {}
distributions = {}
rfraction = {}

for varname in varnames:
    isalin = np.where(ncid.variables['salinity'][:] == salinities[varname])
    means[varname] = np.zeros((nperiods,12))
    stddevs[varname] = np.zeros((nperiods,12))
    npts[varname] = np.zeros((nperiods,12),dtype='int')
    distributions[varname] = {}
    rfraction[varname] = np.zeros((nperiods,12,2))
    for (iperiod,startyear) in enumerate(dividers[:-1]):

        distributions[varname][startyear] = []
        for month in range(1,13):
            all_index = np.where(np.logical_and(np.logical_and(
                              ncid.variables['year'][:] >= startyear,
                              ncid.variables['year'][:] < dividers[iperiod+1]),
                           ncid.variables['month'][:] == month))

            dist = ncid.variables[varname][all_index[0],isalin[0]]
            nma = np.where(dist.mask != True)
            dist_nma = dist[nma]
            npts[varname].itemset((iperiod,month-1),dist_nma.shape[0])
            means[varname].itemset((iperiod,month-1),np.mean(dist_nma))
            stddevs[varname].itemset((iperiod,month-1),np.std(dist_nma))

            distributions[varname][startyear].append(dist_nma)
    
            for region_code in range(1,3):
                region_all_index = np.where(np.logical_and(np.logical_and(
                    ncid.variables['year'][:] >= startyear,
                    ncid.variables['year'][:] < dividers[iperiod+1]),
                    np.logical_and(
                    ncid.variables['month'][:] == month,
                    ncid.variables['region_code'][:] == region_code)))
                
                try:
                    dist_r = ncid.variables[varname][region_all_index[0],
                                                     isalin[0]]
                    nma = np.where(dist_r.mask != True)
                    dist_r_nma = dist_r[nma]
                    rfraction[varname].itemset((iperiod,month-1,region_code-1),
                        dist_r_nma.shape[0] / dist_nma.shape[0])
                except IndexError:
                    rfraction[varname].itemset((iperiod,month-1,region_code-1),
                        0)
                    
    
    
xx_plot = np.arange(.3,12,1)
offset = .2
width = .1
            
xl = .06
yb = .55
xw = .4
yw = .4
ys = .5
xs = .5

figure = plt.figure(figsize=(12,12))

for (ivar,varname) in enumerate(varnames):
    xi = ivar % 2
    yi = ivar // 2
    axis = figure.add_axes([xl+xs*xi,yb-ys*yi,xw,yw])

    for (iperiod,startyear) in enumerate(dividers[:-1]):
        period_label = str(startyear)+'-'+str(dividers[iperiod+1]-1)
        bp = axis.boxplot(distributions[varname][startyear],positions=xx_plot+offset*iperiod,
                          widths=width,whis=10)
        for item in ['boxes','whiskers','fliers','means','medians','caps']:
            plt.setp(bp[item],color=colours[startyear])

        axis.plot(xx_plot+offset*iperiod,means[varname][iperiod,:],
                  markerfacecolor=colours[startyear],
                  marker='o',markeredgecolor='k',linestyle=' ')
        axis.plot([0],[0],color=colours[startyear],label=period_label)

        #for month in range(1,13):
            #axis.text(xx_plot[month-1]+offset*iperiod,
                      #np.max(distributions[varname][startyear][month-1])+1,
                      #str(int(npts[varname][iperiod,month-1])),
                      #ha='center')

    for month in range(1,13):
        axis.plot([month,month],ylims[varname],color='#e0e0e0',linestyle='--')
        axis.text(month-.5,
            ylims[varname][0]-.04*(ylims[varname][1]-ylims[varname][0]),
            str(month),ha='center')

    for label in axis.get_xticklabels():
        label.set_visible(False)

    axis.set_ylim(ylims[varname])
    axis.set_xlim(0,12)
    axis.set_xticks(np.arange(0,13))
    axis.set_ylabel(var_titles[varname]+' ($Wm^2$)')
    axis.plot([0,12],[0,0],linestyle='--',color='k')

axis.legend()

pylab.savefig(outfile)

