# This makes use of the open source Iris library

import sys
import matplotlib.pyplot as plt
sys.path.insert(0,) # Insert reference to location of IMB basic processing code
                    # which is available at 
                    # https://github.com/sammiebuzzard/IMBS_MO
import my_analysis_new as ma_new
import calculate
import bfiles
import glob
import copy
import iris.coord_categorisation as icc
import datetime as dt 
import numpy as np
import regions
from arrays import wrap
from matplotlib.font_manager import FontProperties
import pylab
import iris
import cube_magic as cm
import netCDF4 as nc4
import stats

font_large = FontProperties(size=18)
buoy_file =  # Set obs input file: a netCDF4 file containing a dataset of monthly mean energy flux data from IMBs
model_files = {} # Set to a dictionary whose keys are 'experiments' below and
                 # whose values are a list of model data files containing 
                 # model ice energy flux and ice concentration data

model = 'HadGEM2_ES'
experiments = ['Hist1','Hist2','Hist3','Hist4']
region_names = ['North_Pole','Beaufort_Sea']
period = [1980,1999]
subdirs = ['ICE','ICE_STATE']

varnames = ['Topmelt','Fcondtop','Fcondbot','Ocean_heat']



all_aice = {experiment: calculate.calc_3D(model_files[experiment],'Ice_area',example_files = \
    bfiles.cpt_subset(in_files))[0] for experiment in experiments}

model_means = {'North_Pole':{},'Beaufort_Sea':{}}
model_stds = {'North_Pole':{},'Beaufort_Sea':{}}
model_percentiles = {'North_Pole':{},'Beaufort_Sea':{}}


for varname in varnames:

    all_fields = {experiment: calculate.calc_3D(model_files[experiment],varname,example_files = \
                                                bfiles.cpt_subset(in_files),force_units = (varname=='Ocean_heat'))[0] for experiment in experiments}
    local_fields = {experiment: all_fields[experiment] / all_aice[experiment] for experiment in experiments}

    for region_name in region_names:
        model_means[region_name][varname] = np.zeros(12)
        model_stds[region_name][varname] = np.zeros(12)
        model_percentiles[region_name][varname] = np.zeros((12,5))
        region_fields = copy.deepcopy(local_fields)
        for experiment in experiments:
            regions.reduce_to_region(region_fields[experiment],region_name,'hadgom')
            icc.add_month(region_fields[experiment],'time')

        for month in range(1,13):
            month_name = dt.date(1980,month,1).strftime('%b')
            data_list = []
            area_list = []
            for experiment in experiments:
                month_index = np.where(region_fields[experiment].coord('month').points == month_name)
                mlfc = region_fields[experiment][month_index[0],:,:]
                cm.llbounds(mlfc)
                areas = iris.analysis.cartography.area_weights(mlfc)
                index = np.where(mlfc.data.mask!=True)
                data_array = mlfc.data.data[index]
                area_array = areas[index]
    
                data_list = data_list + list(data_array)
                area_list = area_list + list(area_array)
    
            all_data_array = np.array(data_list)
            all_area_array = np.array(area_list)
               
            data_mean = np.sum(all_data_array * all_area_array) / np.sum(all_area_array)
            data_std = np.sqrt(np.sum(all_data_array **2. * all_area_array)/np.sum(all_area_array)-\
                        data_mean ** 2.)

            data_percentiles = stats.weighted_percentiles(all_data_array,\
                                                          all_area_array,\
                                                          percentiles=[1.,25.,50.,75.,99.])

            model_means[region_name][varname].itemset(month-1,data_mean)
            model_stds[region_name][varname].itemset(month-1,data_std)
            model_percentiles[region_name][varname][month-1,:] = \
                data_percentiles


regions_dic = {1: 'North_Pole', 2: 'Beaufort_Sea'}

varnames_plot = ['top_melt','fcondtop','fcondbot','ocean_heat_flux']
salinities = {'top_melt':0.,'fcondtop':1.,'fcondbot':4.,'ocean_heat_flux':4.}

imb_means = {}
imb_stds = {}
imb_percentiles = {}
ncid = nc4.Dataset(buoy_file)
for region_code in range(1,3):
    region_name = regions_dic[region_code]
    imb_means[region_name] = {}
    imb_stds[region_name] = {}
    imb_percentiles[region_name] = {}
    for varname in varnames_plot:
        isalin = int(salinities[varname] * 2)
        imb_means[region_name][varname] = np.zeros(12)
        imb_stds[region_name][varname] = np.zeros(12)
        imb_percentiles[region_name][varname] = np.zeros((12,5))
        
        for month in range(1,13):
            index = np.where(np.logical_and(ncid.variables['month'][:]==month,\
                              ncid.variables['region_code'][:]==region_code))
            dist = ncid.variables[varname][index[0],isalin]
            nma = np.where(dist.mask == False)
            dist_nma = dist[nma]
            imb_means[region_name][varname].itemset(month-1,np.mean(dist_nma))
            imb_stds[region_name][varname].itemset(month-1,np.std(dist_nma))
            for (ipc,percentile) in enumerate([1,25,50,75,99]):
                imb_percentiles[region_name][varname].itemset((month-1,ipc), 
                    np.percentile(dist_nma,percentile))


imb_boxes_list = {region_name:
                  {varname:
                   [
                    {
        'label' : 'IMB estimates'*(imonth==0),
        'whislo': imb_percentiles[region_name][varname][imonth,0],
        'q1'    : imb_percentiles[region_name][varname][imonth,1],
        'med'   : imb_percentiles[region_name][varname][imonth,2],
        'q3'    : imb_percentiles[region_name][varname][imonth,3],
        'whishi': imb_percentiles[region_name][varname][imonth,4],
        'fliers': []
                    }
                    for imonth in range(0,12)]
                   for varname in varnames_plot}
                  for region_name in region_names}

model_boxes_list = {region_name:
                  {varname:
                   [
                    {
        'label' : 'HadGEM2-ES'*(imonth==0),
        'whislo': model_percentiles[region_name][varname][imonth,0],
        'q1'    : model_percentiles[region_name][varname][imonth,1],
        'med'   : model_percentiles[region_name][varname][imonth,2],
        'q3'    : model_percentiles[region_name][varname][imonth,3],
        'whishi': model_percentiles[region_name][varname][imonth,4],
        'fliers': []
                    }
               for imonth in range(0,12)]
                   for varname in varnames}
                    for region_name in region_names}

outfile_eps = # Set location of output file

varnames_imb_dic = {'Topmelt':'top_melt',\
                    'Fcondtop':'fcondtop',\
                    'Fcondbot':'fcondbot',\
                    'Ocean_heat':'ocean_heat_flux'}
varnames_title_dic = {'Topmelt':'Top melting flux',\
                    'Fcondtop':'Top conductive flux',\
                    'Fcondbot':'Basal conductive flux',\
                    'Ocean_heat':'Ocean heat flux'}
xl = .12
yb = .77
yw = .18
xw = .37
xgap = .49
ygap = .23

obs_color = 'k'
model_color = 'r'
width = .15

letters = ['(a)','(b)','(c)','(d)','(e)','(f)','(g)','(h)']

xx_plot = np.arange(.5,12,1)
offset = .2
figure = plt.figure(figsize=(9,15))
for (ivar,varname) in enumerate(varnames):
    vi = varnames_imb_dic[varname]
    for (iregion,region_name) in enumerate(region_names):
        axis = figure.add_axes([xl+xgap*iregion, yb-ygap*ivar,xw,yw])
        bp = axis.bxp(imb_boxes_list[region_name][vi],
                 positions=xx_plot+offset,
                 widths=width)
        for item in ['boxes','whiskers','fliers','means','medians','caps']:
            plt.setp(bp[item],color=obs_color)
        
        bp = axis.bxp(model_boxes_list[region_name][varname],
                 positions=xx_plot-offset,
                 widths=width)
        for item in ['boxes','whiskers','fliers','means','medians','caps']:
            plt.setp(bp[item],color=model_color)
        
        axis.plot(xx_plot+offset,imb_means[region_name][vi],
                  marker='o',markerfacecolor=obs_color,
                  markeredgecolor='k',linestyle=' ')
        
        axis.plot(xx_plot-offset,model_means[region_name][varname],
                  marker='o',markerfacecolor=model_color,
                  markeredgecolor='k',linestyle=' ')
        
        axis.set_xlim(0,12)
        axis.set_xticks(np.arange(0,13))
        ylim = axis.get_ylim()
        for month in range(1,13):
            axis.plot([month,month],ylim,color='#e0e0e0')
            axis.text(month-.5,ylim[0] - .12*(ylim[1]-ylim[0]),\
                      str(month),ha='center')
        
        axis.set_xlabel('Month',labelpad=15)
        for label in axis.get_xticklabels():
            label.set_visible(False)
        
        axis.set_ylabel(varnames_title_dic[varname]+'($Wm^{-2}$)')
        axis.set_title(letters[iregion + ivar*2])
        axis.plot([0,12],[0,0],color='k',linestyle='--')
        if ivar == 0:
            axis.text(6,ylim[1] + .15*(ylim[1]-ylim[0]),\
                      region_name.replace('_',' '),ha='center',\
                      fontproperties = font_large)

axis.errorbar([-1],[0],yerr=[0],label='HadGEM2-ES',\
    color = model_color)
axis.errorbar([-1],[0],yerr=[0],label='IMB estimates',\
    color = obs_color)
axis.set_xlim(0,12)

axis.legend(bbox_to_anchor = [-.1,-.22])
pylab.savefig(outfile)
