import os
import glob
import numpy as np
import numpy.ma as ma
from arrays import wrap
import matplotlib.pyplot as plt
import pylab
import datetime as dt
import sys
import itd

outfile = # Set location of output file

datadir = # Set location of NSIDC submarine ice thickness data available at https://nsidc.org/data/g01360

subs = os.listdir(datadir)

itd_list = []
for (isub,sub) in enumerate(subs):
    print isub, sub
    fulldir = datadir + sub + '/'
    files = glob.glob(fulldir + '*stat*')
    for ffile in files:
        iitd = itd.itd()
        iitd.read(ffile)
        if iitd.validate():
            itd_list.append(iitd)

months = [iitd.month for iitd in itd_list]
years = [iitd.year for iitd in itd_list]
rcs = [iitd.spatially_classify() for iitd in itd_list]

itd_list_np = [iitd for iitd in itd_list if iitd.spatially_classify() == 1]
itd_list_bs = [iitd for iitd in itd_list if iitd.spatially_classify() == 2]

np_aggregates = []
bs_aggregates = []

np_nyears = []
bs_nyears = []
for month in range(1,13):
    itd_month_list_np = [iitd for iitd in itd_list_np if iitd.month == month]
    itd_month_list_bs = [iitd for iitd in itd_list_bs if iitd.month == month]
    
    np_aggregate = itd.aggregate(itd_month_list_np)
    bs_aggregate = itd.aggregate(itd_month_list_bs)

    np_aggregates.append(np_aggregate)
    bs_aggregates.append(bs_aggregate)

    np_nyear = np.unique(np.array([\
           iitd.year for iitd in itd_month_list_np])).shape[0]
    bs_nyear = np.unique(np.array([\
           iitd.year for iitd in itd_month_list_bs])).shape[0]

    np_nyears.append(np_nyear)
    bs_nyears.append(bs_nyear)

cat_bnds = [0,6,14,24,36,200]

model_itd_np = ma.masked_array(np.zeros((12,5)),\
    mask=np.zeros((12,5),dtype='bool'))
model_itd_bs = ma.masked_array(np.zeros((12,5)),\
    mask=np.zeros((12,5),dtype='bool'))
for month in range(1,13):
    for cat in range(1,6):
        if np_nyears[month-1] == 0:
            model_itd_np.mask.itemset((month-1,cat-1),True)
        else:
            total_fraction = \
              np.sum(np_aggregates[month-1][1][cat_bnds[cat-1]:cat_bnds[cat]])
            model_itd_np.data.itemset((month-1,cat-1),total_fraction)
        
        if bs_nyears[month-1] == 0:
            model_itd_bs.mask.itemset((month-1,cat-1),True)
        else:
            total_fraction = \
              np.sum(bs_aggregates[month-1][1][cat_bnds[cat-1]:cat_bnds[cat]])
            model_itd_bs.data.itemset((month-1,cat-1),total_fraction)


model_itd_np_smooth = np.zeros((12,5))
model_itd_bs_smooth = np.zeros((12,5))

for cat in range(1,6):
    smooth_sc = itd.seasonal_cycle_smooth(model_itd_np.data[:,cat-1],np.array(np_nyears))
    model_itd_np_smooth[:,cat-1] = smooth_sc
    smooth_sc = itd.seasonal_cycle_smooth(model_itd_bs.data[:,cat-1],np.array(bs_nyears))
    model_itd_bs_smooth[:,cat-1] = smooth_sc

xl = .11
yb = .22
yw = .76
xw = .88
xgap = .33

xx_plot = np.arange(-.5,13)

colours = ['r','g','c','b','k']
figure = plt.figure(figsize=(7,7))
ax = figure.add_axes([xl+xgap*0,yb,xw,yw])

for cat in range(1,6):
    ax.plot(xx_plot,wrap(model_itd_np_smooth[:,cat-1]),\
        color = colours[cat-1],marker='*',\
        label=str(cat),linestyle='-')
    ax.plot(xx_plot,wrap(model_itd_bs_smooth[:,cat-1]),\
        color = colours[cat-1],marker='*',\
        linestyle='--')

ax.set_ylabel('Ice fraction in thickness category')

ax.plot([0],[0],color='#ffffff',label=' ')
ax.plot([0],[0],linestyle='-',label='North Pole',color='k')
ax.plot([0],[0],linestyle='--',label='Beaufort Sea',color='k')

ax.legend(bbox_to_anchor = [.2,-.58,.5,.5],ncol=3)
ax.set_xlim(0,12)

ax.set_xticks(np.arange(0,13))
ylim = ax.get_ylim()
for label in ax.get_xticklabels():
    label.set_visible(False)

for month in range(1,13):
    month_name = dt.date(1980,month,1).strftime('%b')[0]
    ax.text(month-.5,ylim[0]-.04*(ylim[1]-ylim[0]),month_name,ha='center')    

pylab.savefig(outfile)

