main_datadir = # Directory where data is held


import datetime as dt
month_names = [dt.date(1980,month,1).strftime('%b').lower() for \
    month in range(1,13)]

class itd:
    def __init__(self):
        import numpy as np
        self.latitude = np.zeros(2)
        self.longitude = np.zeros(2)
        self.lower_bounds = []
        self.fraction = []

    def read(self,datafile):
        fileh = open(datafile)

        mode = 'metadata'
        for line in fileh.readlines():
            r = decipher(line)
            if r is None:
                if mode == 'metadata':
                    keyval = line.split(':')
                    if len(keyval) == 2:
                        if keyval[0].strip()=='Beginning Latitude':
                            self.latitude[0] = float(keyval[1])
                        if keyval[0].strip()=='Beginning Longitude':
                            self.longitude[0] = float(keyval[1])
                        if keyval[0].strip()=='Ending Latitude':
                            self.latitude[1] = float(keyval[1])
                        if keyval[0].strip()=='Ending Longitude':
                            self.longitude[1] = float(keyval[1])
                        if keyval[0].strip()=='Month':
                            self.month_name = keyval[1].strip()
                            self.month = month_names.index(\
                             self.month_name.lower())+1
                        if keyval[0].strip()=='Year':
                            self.year = float(keyval[1])
                else:
                    mode = 'completed'
            else:
                if mode in ['data','metadata']:
                    mode = 'data'
                    self.lower_bounds.append(r[1])
                    self.fraction.append(r[2])
        
    def validate(self):
        if not hasattr(self,'month_name') or \
            not hasattr(self,'year'):
            return False
        else:
            return True
           

    def spatially_classify(self):
        if in_north_pole_region([self.longitude[0],self.latitude[0]]) and \
           in_north_pole_region([self.longitude[1],self.latitude[1]]):
            return 1
        if in_beaufort_sea_region([self.longitude[0],self.latitude[0]]) and \
           in_beaufort_sea_region([self.longitude[1],self.latitude[1]]):
            return 2
 
        return 0


def in_north_pole_region(coord):
    lon, lat = coord
    return ((lat > 85.) or (lat > 81. and (lon > 320 or lon < 40)))

def in_beaufort_sea_region(coord):
    lon, lat = coord
    return ((lat < 80 and lat > 72) and \
      ((lon > 180 and lon < 235) or (lon > -180 and lon < -125)))


def decipher(line):
    if ',' in line:
        elements = line.split(',')
        if len(elements) >= 3:
            try:
                bin_no = int(elements[0])
                lower_bound = float(elements[1])
                fraction = float(elements[2])
            except ValueError:
                return None
        else:
            return None
    else:
        if len(line) >= 23:
            try:
                bin_no = int(line[:5])
                lower_bound = float(line[5:11])
                fraction = float(line[11:23])
            except ValueError:
                return None
        else:
            return None

    return (bin_no,lower_bound,fraction)


def aggregate(itd_list,month=None,year=None,region_code=None):
    import numpy as np
    import copy

    itd_sublist = copy.deepcopy(itd_list)

    if month is not None:
        itd_sublist = [iitd for iitd in itd_sublist if iitd.month == month]

    if year is not None:
        itd_sublist = [iitd for iitd in itd_sublist if iitd.year == year]

    if region_code is not None:
        itd_sublist = [iitd for iitd in itd_sublist if \
            iitd.spatially_classify() == region_code]

    accounting_dic = {}
    for iitd in itd_sublist:
        for (lb,fr) in zip(iitd.lower_bounds,iitd.fraction):
            try:
                accounting_dic[lb].append(fr)
            except KeyError:
                accounting_dic[lb] = [] 
                accounting_dic[lb].append(fr)

    lower_bounds = accounting_dic.keys()
    lower_bounds.sort()

    fractions = []
    for key in lower_bounds:
        total_fraction = np.sum(np.array(accounting_dic[key]))
        fractions.append(total_fraction)

    fractions = np.array(fractions)
    fractions = fractions / np.sum(fractions)

    return (lower_bounds,fractions)

        
def seasonal_cycle_smooth(sc_source,sc_weight):
    import numpy as np
    base_weights = np.array([1,4,6,4,1]).astype('float')
    out_data = np.zeros(12)
    for month in range(1,13):
        source_indices = np.array([(ii+month-1) % 12 for ii in range(-2,3)])
        source_array = sc_source[source_indices]
        weight_array = sc_weight[source_indices]

        complete_weights = base_weights * weight_array
        datapoint = np.sum(source_array * complete_weights) / \
                    np.sum(complete_weights)

        print datapoint
        out_data.itemset(month-1,datapoint)

    return out_data


def get_full_list():
    import os
    import glob
    datadir = main_datadir
    subs = os.listdir(datadir)

    itd_list = []
    for (isub,sub) in enumerate(subs):
        print isub, sub
        fulldir = datadir + sub + '/'
        files = glob.glob(fulldir + '*stat*')
        for ffile in files:
            iitd = itd()
            iitd.read(ffile)
            if iitd.validate():
                itd_list.append(iitd)
    
    return itd_list


def get_sub_list(itd_list,month=None,year=None,region_code=None):
    import copy
    months = [iitd.month for iitd in itd_list]
    years = [iitd.year for iitd in itd_list]
    rcs = [iitd.spatially_classify() for iitd in itd_list]

    sub_list = copy.deepcopy(itd_list)    
    if month is not None:
        sub_list = [iitd for iitd in sub_list if iitd.month == month]

    if year is not None:
        sub_list = [iitd for iitd in sub_list if iitd.year == year]

    if region_code is not None:
        sub_list = [iitd for iitd in sub_list if iitd.spatially_classify() \
                         == region_code]

    return sub_list


def total_distribution(smoothed=False):
    import numpy as np
    import numpy.ma as ma
    cat_bnds = [0,6,14,24,36,200]
    itd_list = get_full_list()

    fraction = ma.masked_array(np.zeros((5,12,2)),\
                   mask = np.zeros((5,12,2),dtype='bool'))
    n_years = np.zeros((12,2))
    for region_code in range(2):
        for month in range(1,13):
            sub_list = get_sub_list(itd_list,month=month,\
                region_code=region_code)
            list_aggregate = aggregate(sub_list)
            list_nyears = np.unique(np.array([\
                iitd.year for iitd in sub_list])).shape[0]
            n_years.itemset((month-1,region_code),list_nyears)

            for cat in range(1,6):
                if list_nyears > 0:
                    total_fraction = \
                      np.sum(list_aggregate[1][cat_bnds[cat-1]:\
                                                        cat_bnds[cat]])
                    fraction.data.itemset((cat-1,month-1,region_code),\
                          total_fraction)
                else:
                    fraction.mask.itemset((cat-1,month-1,region_code),True)

    if smoothed:
        smoothed_array = np.zeros((5,12,2))
        for region_code in range(2):
            for cat in range(1,6):
                source_array1D = fraction.data[cat-1,:,region_code]
                smooth_array1D = seasonal_cycle_smooth(source_array1D,\
                     n_years[:,region_code])
                smoothed_array[cat-1,:,region_code] = smooth_array1D

        return smoothed_array
    else:
        return fraction
